library ieee;
use ieee.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_unsigned.all;

entity divider is
    port(B,A: in std_logic_vector(7 downto 0);
        R,Q: out std_logic_vector(7 downto 0);
        CLK: in std_logic);
    end divider;
    
    architecture divider_arch of divider is
        
        function CONV_INTEGER (X: STD_LOGIC_VECTOR) return INTEGER is
        variable RESULT: INTEGER;
        begin
        RESULT :=0;
        for i in X'range loop
        RESULT := RESULT*2;
        case X(i) is
        when '0'|'L' => null;
        when '1'|'H' => RESULT :=RESULT +1;
        when others => null;
        end case;
        end loop;
        return RESULT;
        end CONV_INTEGER;
        
        signal LR,LC,ER,EA,Rsel,EQ,EC,EB,LA,wq,count: std_logic:='0';
        signal Rtemp,IQ: std_logic_vector(7 downto 0);
        signal counterout: std_logic_vector(3 downto 0):= (others => '0');
        signal counterin: std_logic_vector(3 downto 0):= (others => '1');        
        signal outb: std_logic_vector(7 downto 0):= (others => '0');
        signal outbn: std_logic_vector(7 downto 0):= (others => '0');
        signal outa: std_logic_vector(7 downto 0):= (others => '0');
        signal outr: std_logic_vector(7 downto 0):= (others => '0');
        signal outmulti: std_logic_vector(7 downto 0):= (others => '0');
        signal adderout: std_logic_vector(7 downto 0):= (others => '0');
                
        component adder port(x,y:in std_logic_vector(7 downto 0);
        c0:in std_logic; c8:out std_logic;
        s:out std_logic_vector(7 downto 0));
        end component;
    
        component Aregister port(LA:in std_logic;EA:in std_logic;
        CLK:in std_logic;A:in std_logic_vector(7 downto 0);
        Q:out std_logic);
        end component;
    
        component Bregister port (CLK:in std_logic;EB:in std_logic; 
        B: in std_logic_vector(7 downto 0);         
        Q: out std_logic_vector(7 downto 0));             
        end component;
        
        component counter port (CLK: in STD_LOGIC; LC: in STD_LOGIC;
        EC: in STD_LOGIC; D: in STD_LOGIC_VECTOR (3 downto 0);
        Q: out STD_LOGIC_VECTOR (3 downto 0);
        count: out STD_LOGIC);
        end component;
        
        component multiplexer port(Rsel: in std_logic;
        Rsubtract: in std_logic_vector(7 downto 0);
        Q: out std_logic_vector(7 downto 0));
        end component;
        
        component quotient port(w,EQ,CLK: in std_logic;
        Q: out std_logic_vector(7 downto 0)); 
        end component;
        
        component Rregister port (LR,ER,CLK,w: in std_logic;
        R:in std_logic_vector(7 downto 0); Q:out std_logic);
        end component;  
    
        
        begin
            U1: Bregister port map(CLK,EB,B,outb);
            U2: Aregister port map(LA,EA,CLK,A,outa(0));
            U3: Rregister port map(wq,ER,CLK,outa(0),outmulti,outr(0));
            U4: multiplexer port map(Rsel,adderout,outmulti);
            outbn<=not outb;
            outr(7 downto 1)<=outr(6 downto 0);
            outa(7 downto 1)<=outa(6 downto 0); 
            U5: adder port map(outbn,outr,'1',wq,adderout);
            U6: quotient port map(wq,EQ,CLK,IQ);
            U7: counter port map(CLK,LC,EC,counterin,counterout,count);
            
            process(counterout,CLK,count)
                begin
                if counterout=7 and count='0' then LC<='1';LR<='1';Rsel<='0';
                else LC<='0';
            end if;
                if counterout=0 and count='1' then LC<='1';LR<='1';Rsel<='0';
                elsif counterout>0 then Rsel<='1';
            end if;
        if CLK'event and CLK='1' then
                if counterout=7 then EB<='1';LA<='1';LR<='1';
                elsif counterout=0 then EB<='0';LA<='0';LR<='0';
                elsif counterout/=7 and counterout/=0 then EB<='1';LA<='1';LR<='1';
            end if;
        
         -- ER<='1';EA<='1';EQ<='1';
            if outr<outr-B then IQ(CONV_INTEGER(counterout))<='0';ER<='1';EA<='1';EQ<='1';
            else IQ(CONV_INTEGER(counterout))<='1';ER<='1';EA<='1';EQ<='1';
        end if;
        end if;
    --  ER<='0';EA<='0';EQ<='0';
    --    end if;
        R<=outr;
        Q<=IQ;                 
    end process;
        end divider_arch;
            