library ieee;
use ieee.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_unsigned.all;

entity divider is
    port(B,A: in std_logic_vector(7 downto 0);
        R,Q: out std_logic_vector(7 downto 0);
        CLK: in std_logic; s: in std_logic);   
    end divider;
    
    architecture divider_arch of divider is 
        
        signal LR,LC,ER,EA,Rsel,EQ,EC,EB,LA,wq,count: std_logic:='0';
        signal Rtemp,IQ: std_logic_vector(7 downto 0);
        signal counterout: std_logic_vector(3 downto 0):= (others => '0');               
        signal outb: std_logic_vector(7 downto 0):= (others => '0');
        signal outbn: std_logic_vector(7 downto 0):= (others => '0');
        signal outa: std_logic_vector(7 downto 0):= (others => '0');
        signal outr: std_logic_vector(7 downto 0):= (others => '0');
        signal outmulti: std_logic_vector(7 downto 0):= (others => '0');
        signal sum: std_logic_vector(7 downto 0):= (others => '0');
      
        
        component adder port(x,y:in std_logic_vector(7 downto 0);
        c0:in std_logic; c8:out std_logic;
        sum:out std_logic_vector(7 downto 0));
        end component;          
      
        component Aregister port(LA:in std_logic;EA:in std_logic;
        CLK:in std_logic;A:in std_logic_vector(7 downto 0);
        Q:out std_logic_vector(7 downto 0));
        end component;
    
        component Bregister port (CLK:in std_logic;EB:in std_logic; 
        B: in std_logic_vector(7 downto 0);         
        Q: out std_logic_vector(7 downto 0));             
        end component;
        
        component counters port (CLK: in STD_LOGIC; LC: in STD_LOGIC;
        EC: in STD_LOGIC; Q: out STD_LOGIC_VECTOR (3 downto 0);
        count: out STD_LOGIC);
        end component;
        
        component multiplexer port(Rsel: in std_logic;
        Rsubtract: in std_logic_vector(7 downto 0);
        Q: out std_logic_vector(7 downto 0));
        end component;
        
        component quotient port(w,EQ,CLK: in std_logic;
        Q: out std_logic_vector(7 downto 0)); 
        end component;
        
        component Rregister port (LR,ER,CLK,w: in std_logic;
        R:in std_logic_vector(7 downto 0); Q:out std_logic_vector(7 downto 0));
        end component;  
        
        component state_machine port(s: in std_logic;
        z: in std_logic; cout: in std_logic; CLK: in std_logic;
        Rsel: out std_logic; LR: out std_logic; LC: out std_logic;
        ER: out std_logic; EA: out std_logic; EQ: out std_logic;
        EC: out std_logic);
        end component;    
        
        begin
            EB<='1' when counterout=15;
            U1: Bregister port map(CLK,EB,B,outb);
            U2: Aregister port map(LA,EA,CLK,A,outa);
            U3: Rregister port map(LR,ER,CLK,outa(7),outmulti,outr);
            U4: multiplexer port map(Rsel,sum,outmulti);
            outbn<=not outb;    
            U5: adder port map(outbn,outr,'1',wq,sum);
            U6: quotient port map(wq,EQ,CLK,IQ);
            U7: counters port map(CLK,LC,EC,counterout,count);
            U8: state_machine port map(s,count,wq,CLK,Rsel,LR,LC,ER,EA,EQ,EC);
            Q<=IQ;
            R<=outr;
        end divider_arch;
            